import os
from pathlib import Path

import axipy
from PySide2.QtCore import Qt
from PySide2.QtGui import QIcon
from PySide2.QtWidgets import QDialog, QFileDialog
from axipy import Notifications

from spatialRound.ui.dlgrunner import DlgRunPrigress
from spatialRound.ui.tools.addTools import load_ui


class DlgCoordinateRound(QDialog):
    def __init__(self,parent=None):
        super(DlgCoordinateRound, self).__init__(parent)
        path_ui = os.path.join(os.path.dirname(__file__), "DlgBaseRound_2.ui")
        self.window=load_ui(path_ui)
        self.window.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
        self.window.cmb_tables.currentIndexChanged.connect(self.__change_table_source)
        curent_path = str(Path(__file__).parent)
        self.__path_icon = os.path.join(curent_path, "icons")
        self.window.tlb_saveas.setIcon(QIcon(os.path.join(self.__path_icon, "saveas.png")))
        self.window.tlb_saveas.clicked.connect(self.__save)
        self.window.pb_cancel.clicked.connect(self.__close)
        self.window.pb_run.clicked.connect(self.__run)
        #self.__load_tab_list()
    def __load_tab_list(self):
        self.window.cmb_tables.clear()
        tables = axipy.da.data_manager.tables
        for tab in tables:
            self.window.cmb_tables.addItem(tab.name)
        self.window.cmb_tables.setCurrentIndex(0)
    def __change_table_source(self,index):
        tab_select=axipy.da.data_manager.find(self.window.cmb_tables.currentText ())
        #path_source=tab_select.properties['tabFile']
        try:
            path_source = tab_select.properties['tabFile']
        except:
            path_source = tab_select.properties['fileName']
        path_src=Path(path_source)
        path_dest=path_src.parent.joinpath(path_src.stem+"_round_coordinate.tab")
        self.window.ln_out_file.setText(str(path_dest))
        self.window.ln_out_file.setCursorPosition(0)
    def __save(self):
        filename = QFileDialog.getSaveFileName(self.window, 'Выбрать файл для сохранения результата',
                                               self.window.ln_out_file.text(), "MapInfo (*.tab)")
        if filename[0] == '':
            return
        self.window.ln_out_file.setText(filename[0])
        self.window.ln_out_file.setCursorPosition(0)
    def __close(self):
        self.window.close()
    def __run(self):
        params = {}
        params['source_tab'] = self.window.cmb_tables.currentText()
        params['decimate'] = self.window.spb_round.value()
        params['out'] = self.window.ln_out_file.text()

        name_tab = str(Path(params['out']).stem)
        tab = axipy.da.data_manager.find(name_tab)
        if tab is not None:
            Notifications.push('Предупреждение', 'Таблица с именем ' + name_tab + ' будет закрыта',
                                   Notifications.Warning)
            tab.close()
        self.window.pb_cancel.setEnabled(False)
        self.window.pb_run.setEnabled(False)
        try:
            cls_prunProgress=DlgRunPrigress("Округление координат",self.window)
            #cls_prunProgress.exec_(params)
            cls_prunProgress.show(params)
        except Exception as ex:
            pass
        self.window.pb_cancel.setEnabled(True)
        self.window.pb_run.setEnabled(True)
        self.window.close()
    def show(self):
        self.__load_tab_list()

        self.window.show()
