import axipy
from PySide2.QtCore import QObject, Signal
from axipy import *
from axipy import provider_manager
from loguru import logger
from osgeo import ogr

def calcResolution(digits):
    if digits==0 or digits is None:
        return 1.
    result = 1.0
    if digits<0:
        for i in range(abs(digits)):
            result=result*10
        return result
    result=1.0
    for i in range(digits):
        result=result/0.1
    return result
def simpleArrayObj(points,decimal,optimization=True):
    new_points=[]
    for pt in points:
        pt_new=Pnt(round(pt.x,decimal),round(pt.y,decimal))
        if not optimization:
            continue
        if len(new_points)==0:
            new_points.append(pt_new)
            continue
        if pt_new.x== new_points[-1].x and pt_new.y== new_points[-1].y:
            continue
        new_points.append(pt_new)
    return new_points
def deleteDouble(points):
    points_out=[]
    for pt in points:
        if len(points_out)==0:
            points_out.append(pt)
            continue
        if points_out[-1].x==pt.x and points_out[-1].y==pt.y:
            continue
def copyGeometryOgr(source_geometry,resolution):
    if source_geometry is None:
        return None
    ogr_geometry=ogr.CreateGeometryFromWkb(source_geometry.to_wkb())
    if ogr_geometry is None:
        logger.debug("Error transform to Ogr ")
        return source_geometry
    #resolution=calcResolution(decimal)
    opts = ["FORMAT=GML3", "SRSNAME_FORMAT=SHORT","XY_COORD_RESOLUTION="+str(resolution)]

    geo_gml=ogr_geometry.ExportToGML(opts)
    new_ogr_geo=ogr.CreateGeometryFromGML(geo_gml)
    new_axi_geo=axipy.Geometry.from_wkb(new_ogr_geo.ExportToWkb(),source_geometry.coordsystem)
    geo_gml=None
    ogr_geometry=None
    new_ogr_geo=None
    return new_axi_geo
def copyGeometry(source_geometry,decimal):
    if source_geometry.type==GeometryType.Point:
        new_geo=Point(round(source_geometry.x,decimal),round(source_geometry.y,decimal),source_geometry.coordsystem)
        return new_geo
    if source_geometry.type == GeometryType.Line:
        pt_1=Pnt(round(source_geometry.begin.x,decimal),round(source_geometry.begin.y,decimal))
        pt_2 = Pnt(round(source_geometry.end.x, decimal), round(source_geometry.end.y, decimal))
        new_geo=Line(pt_1,pt_2,source_geometry.coordsystem)
        return new_geo
    if source_geometry.type==GeometryType.LineString :
        new_geo=LineString(simpleArrayObj(source_geometry.points,decimal),source_geometry.coordsystem)
        return new_geo
    if source_geometry.type==GeometryType.Polygon :
        new_geo=Polygon(simpleArrayObj(source_geometry.points,decimal),source_geometry.coordsystem)
        if len(source_geometry.holes)>0:
            for pnts_hole in source_geometry.holes:
                polygon_holes=Polygon(simpleArrayObj(pnts_hole,decimal),source_geometry.coordsystem)
                if not polygon_holes.is_valid:
                    polygon_holes=polygon_holes.normalize()
                if polygon_holes is not None:
                    new_geo.holes.append(polygon_holes.points)
        return new_geo
    if source_geometry.type==GeometryType.MultiPoint:
        new_geo=MultiPoint()
        for  pt in source_geometry:
            new_geo.append(copyGeometry(pt,decimal))
        new_geo.coordsystem=source_geometry.coordsystem
        return new_geo
    if source_geometry.type == GeometryType.MultiLineString:
        new_geo = MultiLineString()
        for item_geo in source_geometry:
            new_geo.append(copyGeometry(item_geo, decimal))
        new_geo.coordsystem = source_geometry.coordsystem
        return new_geo
    if source_geometry.type == GeometryType.MultiPolygon:
        new_geo = MultiPolygon()
        for item_geo in source_geometry:
            new_geo.append(copyGeometry(item_geo, decimal))
        new_geo.coordsystem = source_geometry.coordsystem
        return new_geo
    if source_geometry.type == GeometryType.GeometryCollection:
        new_geo = GeometryCollection()
        for item_geo in source_geometry:
            new_item_collection=copyGeometry(item_geo, decimal)
            if new_item_collection is None:
                continue
            new_geo.append(new_item_collection)
        if len(new_geo)==0:
            return None
        new_geo.coordsystem = source_geometry.coordsystem
        return new_geo
    return None
def createCopyTable(table_source,path_dest,dectimate):
    schema=table_source.schema


    # table_dest=provider_manager.create_open(path_out,definition)
    table_dest = provider_manager.tab.create_open(path_dest,  schema)
    return table_dest
def copyFeature(table_source,table_dest,decimate=None):
    resolution=calcResolution(decimate)
    for ft in table_source.items():
        geometry=ft.geometry
        #new_geometry=copyGeometry(geometry,decimate)
        new_geometry = copyGeometryOgr(geometry, resolution)
        ft.geometry=new_geometry
        table_dest.insert(ft)
    table_dest.commit()
    table_dest.close()
class ProcessRunner(QObject):
    updateProcess = Signal(float)
    updateFormatProgressBar = Signal(str)
    finished = Signal(int)
    def __init__(self,parent_cls):
        super(ProcessRunner, self).__init__()
        self.__parent=parent_cls
        self.__old_val_progress = 0
        self.__cancelProcess = False
    def setParams(self,params):
        self.__params=params
    def __process_callback(self, complete):
        # print('progress: {}, message: "{}", unknown {}'.format(complete, message,user_data))
        if self.__cancelProcess:
            return 0
        cur_progress_int=int(complete*100+0.5)
        if cur_progress_int>self.__old_val_progress:
            self.updateProcess.emit(complete * 100)
            self.__old_val_progress=cur_progress_int
        else:
            if cur_progress_int<self.__old_val_progress:
                self.__old_val_progress = cur_progress_int
                self.updateProcess.emit(complete * 100)
        #time.sleep(0.001)
        # self.__parent.window.pr_bar.setValue(complete*100)
        # print(complete*100)
        return 1
    def __isCancel(self):
        self.__cancelProcess=True

    def setCancel(self):
        self.__cancelProcess = True
    def run(self):
        try:
            self.__parent.disconnect(self.__isCancel)
        except:
            pass
        self.__parent.cancelProcess.connect(self.__isCancel)
        self.__cancelProcess = False
        self.updateFormatProgressBar.emit("Округление координат .. %p%")
        self.updateProcess.emit(0)
        table = data_manager.find(self.__params['source_tab'])
        path_dest=self.__params['out']
        decimate=self.__params['decimate']
        tab_dest = createCopyTable(table, path_dest, 0)
        ft_count=table.count()
        for i,ft in enumerate(table.items()):
            is_cancel = self.__process_callback(i / ft_count)
            if is_cancel==0:
                break

            geometry = ft.geometry
            new_geometry = copyGeometry(geometry, decimate)
            ft.geometry = new_geometry
            tab_dest.insert(ft)
        tab_dest.commit()
        self.updateProcess.emit(100)
        self.finished.emit(100)
        return

